#!/bin/sh
# Download and package libreoffice as Fatdog SFS
# (C) James Budiono 2013, 2015
# License: GNU GPL Version 3 or later
# $1 - optional tmpdir
# i18n by L18L
# user's languages on top of list (see 5.1, 5.2, 6. and 6.1)

export TEXTDOMAIN=get_app

# configuration
APPTITLE="$(gettext 'SFS Maker for Libreoffice')"
MASTER_URL=http://download.documentfoundation.org/libreoffice/stable
PKGFORMAT=tar.gz
NEEDED_SPACE=$(( 1500*1024 ))	# in KB - this is 1.5 GB
COMP_MODE="-comp xz -Xbcj x86"
#COMP_MODE="-comp lzo -Xcompression-level 1"

# run-time variables
TMPDIR=/tmp/getlibre
TMPFILE=$(mktemp -p /tmp getlibre.XXXXXXXX)
LATEST=		# the version to download
VERSION=	# the major version (4.0, 3.6, etc)
LANGPACK=	# language pack to download
ROOT_URL=
MAIN_PKG=
SDK_PKG=
HELP_PKG=
LANG_PKG=
LANG_NAMES=/tmp/lang_names  # languages in their names
# 1. must be run on tty and use encodings
if ! [ -t 0 ]; then
	exec rxvt encodings=eu+vn+jp+jp-ext+kr+zh+zh-ext -e "$0" "$@"
fi

# 1.1 check for help
case "$1" in
	--help|-h) echo "$(gettext 'Usage:') ${0##*/} [$(gettext '/path/to/tmpdir')]"
			  echo "$(gettext 'Download Libreoffice and make SFS in tmpdir') ($(gettext 'default:') $TMPDIR)"
			  exit
esac
TMPDIR=${1:-$TMPDIR}

# 2. make tempdir & check free space
mkdir -p "$TMPDIR" 2> /dev/null
cd "$TMPDIR"

AVAIL_SPACE=$(df -k . | awk 'NR==2 { print $4 }')
if [ $AVAIL_SPACE -lt $NEEDED_SPACE ]; then
    AVAIL_SPACE=$(( $AVAIL_SPACE / 1024 ))
    NEEDED_SPACE=$(( $NEEDED_SPACE / 1024 ))
	echo $(eval echo `gettext 'Not enough space in $TMPDIR, available: $AVAIL_SPACE MB, required: $NEEDED_SPACE MB'`)
	exit
fi

# 3. find available versions
echo $(gettext 'Checking available versions ...')
AVAIL_VERSIONS=$(wget -q -O - $MASTER_URL | sed -n "
/href=\"[0-9]\\.[0-9]*\\.[0-9]*\\/\"/ {
	s|[^h]*href=\"\\([0-9]\\.[0-9]*\\.[0-9]*\\)/\".*|\\1|
	p
}") 

# 4. choose versions
pp=""
for p in $AVAIL_VERSIONS; do pp="$pp $p ''"; done
if ! eval dialog --title \"$APPTITLE\" --menu \"$(gettext 'Choose version')\" 0 0 0 $pp 2> $TMPFILE; then
	clear
	echo $(gettext 'Cancelled.')
	rm $TMPFILE
	exit
fi
LATEST=$(cat $TMPFILE)
VERSION=${LATEST%.*}
clear
echo $(eval echo `gettext 'Chosen version: $LATEST'`)

# 4.1 define root URL based on version
ROOT_URL=${MASTER_URL}/${LATEST}/deb/$(uname -m)

# 5. find available languages
echo $(gettext 'Checking available languages ...')
AVAIL_LANGS=$(wget -q -O - $ROOT_URL | sed -n "
/LibreOffice_${LATEST}_Linux_x86-64_deb_langpack_.*.${PKGFORMAT}\"/ {
	s|[^h]*href=\"LibreOffice_${LATEST}_Linux_x86-64_deb_langpack_\\(.*\\).${PKGFORMAT}\".*|\\1|
	p
}" | sort | uniq)

# 5.1 move user's LOCALES to top of list
NUM_LOCALES=`echo $AVAIL_LANGS | wc -w`
myLOCALES="${LANG%.*} `locale -a | grep .utf8 | grep -v ${LANG%.*} | cut -d. -f1 | sort -u | tr '\n' ' '`"

#myLOCALES="frz ar en_US pa_IN bn_IN $myLOCALES" # for test only
mtPRE() { for L in "$@"; do [[ "$AVAIL_LANGS" =~ "$L" ]] && { PRE="$PRE $L" && AVAIL_LANGS="${AVAIL_LANGS//$L}" ; } ;  done; }

PRE=""; for aLOCALE in $myLOCALES
do
 case $aLOCALE in
  bn_IN) mtPRE bn-IN bn       ;; # Bengali for India
  bn_*)  mtPRE bn bn-IN       ;; # Bengali
  ca_ES) mtPRE ca-valencia ca ;; # Catalan for Spain
  ca_*)  mtPRE ca ca-valencia ;; # Catalan
  en_GB) mtPRE en-GB          ;; # English
  en_ZA) mtPRE en-ZA          ;; # English
  en_*)  [[ "$PRE" =~ "en-GB" ]] || mtPRE en-GB
         [[ "$PRE" =~ "en-ZA" ]] || mtPRE en-ZA 
         ;; # English
  km*)   mtPRE kmr-Latn km    ;; # Khmer
  pa_*)  mtPRE pa-IN          ;; # Punjabi(Gurmukhi)
  pt_BR) mtPRE pt-BR          ;; # Brazilian
  sa_*)  mtPRE sa-IN          ;; # Sanskrit
  sr_*)  mtPRE sr-Latn sr     ;; # Serbian
  sw_*)  mtPRE sw-TZ          ;; # Swahili
  zh_CN) mtPRE zh-CN          ;; # Chinese
  zh_TW) mtPRE zh-TW          ;; # Chinese
  zh_*)  mtPRE zh-CN zh-TW    ;; # Chinese
  *)     mtPRE ${aLOCALE%_*}  ;; # anything else is using just the Language Part of locale               
 esac
done
AVAIL_LANGS="$PRE ${AVAIL_LANGS}" # user's LOCALES moved to top of list

# 5.2 get languages' names
if [ ! -f $LANG_NAMES ]; then # extract  them from website
 URL='http://meta.wikimedia.org/wiki'
 SOURCE='Template:List_of_language_names_ordered_by_code'
 cd /tmp
 [ ! -f $SOURCE ] && wget $URL/$SOURCE
 echo "Extracted
from $SOURCE
of $URL
at `date
`
" > $LANG_NAMES
 grep 'xml:lang=' "$SOURCE" | while read L
 do
  L=${L#*:} #ex:lang="tk"><span dir="ltr">Туркмен</span>&#160;/ <span dir="rtl">تركمن</span></td>
  KEY="${L#*\"}"; KEY="${KEY%%\"*}" #ex:tk
  VAL=${L#*\>} ; VAL=${VAL//<\/td>} #ex:<span dir="ltr">Туркмен</span>&#160;/ <span dir="rtl">تركمن</span>
  VAL=${VAL//<span\ dir=\"ltr\">} ;  VAL=${VAL//<span\ dir=\"rtl\">} 
  VAL=${VAL//\&\#160\;/ } ; VAL=${VAL//<\/span>}
  echo "${KEY}:${VAL}" >> $LANG_NAMES  #ex:tk:Туркмен / تركمن or another ex:th:ไทย / Phasa Thai
 done
 cd - >/dev/null
fi

# 6. choose languages
pp=""
if [ -f $LANG_NAMES ] ; then # language in language of language
    for p in $AVAIL_LANGS; do
        L="`grep "^$p:" $LANG_NAMES \
          | grep "^${p%-*}:"  $LANG_NAMES`" #ex:'lo:ລາວ / Pha xa lao '
        L=${L#*:}                          #ex:   'ລາວ / Pha xa lao '
        L=${L% *}                          #ex:   'ລາວ / Pha xa lao'
        [ "$L" ] || L=' '                              
      # L=${L// /_}                        #ex:   'ລາວ / Pha_xa_lao'
      # L=${L// /░}                        #ex:   'ລາວ░/░Pha░xa░lao'
        L=${L// /…}                        #ex:   'ລາວ…/…Pha…xa…lao'
        [[ "$L" =~ ":" ]] && L="`echo $L | cut -d ' ' -f1`"
        pp="$pp $p $L" 
    done
    pp=${pp//(/\\(}; pp=${pp//)/\\)} # escape parenthesis ex:nb Norsk_(bokmål)
else
    for p in $AVAIL_LANGS; do pp="$pp $p ''"; done
fi
# echo pp=$pp

> chosenLANG 2>/dev/null
maxNUM=4
while [ $maxNUM -gt 0 ]; do
 TEXT="$(eval echo `ngettext 'Choose one language or Cancel' 'Choose up to $maxNUM languages' $maxNUM`) $(eval echo `ngettext 'or Cancel' 'from these $NUM_LOCALES languages' $NUM_LOCALES`).
 $(gettext 'Cancel only stops choosing language.')
 $(gettext 'Chosen: ') `cat chosenLANG`"
 
 if ! eval dialog --title \"$APPTITLE $LATEST\" --menu \"$TEXT\" 0 0 0 $pp 2> $TMPFILE; then
  rm $TMPFILE
  #exit
  break # stop only choice of languages
 fi
 LANGPACK=$(cat $TMPFILE)
 [ `grep "$LANGPACK" chosenLANG ` ] && continue # avoid double entries
 echo $LANGPACK >> chosenLANG 
 clear 
 rm $TMPFILE
 maxNUM=$(($maxNUM-1))
 NUM_LOCALES=$(($NUM_LOCALES-1))
done

echo $(eval echo `gettext 'Chosen version: $LATEST chosen language: $(cat chosenLANG)'`)

# 6.1 define packages
MAIN_PKG=LibreOffice_${LATEST}_Linux_x86-64_deb
SDK_PKG=LibreOffice_${LATEST}_Linux_x86-64_deb_sdk
while read LINE; do
 [ "$LINE" ] || continue
 HELP_PKG="$HELP_PKG LibreOffice_${LATEST}_Linux_x86-64_deb_helppack_${LINE}"
 LANG_PKG="$LANG_PKG LibreOffice_${LATEST}_Linux_x86-64_deb_langpack_${LINE}"
done < chosenLANG

# 7. get all the stuff
for p in $MAIN_PKG $HELP_PKG $LANG_PKG $SDK_PKG; do
	echo $(eval echo `gettext 'Downloading $p'`)
	if ! wget -nv -c ${ROOT_URL}/${p}.${PKGFORMAT}; then
		echo "$(eval echo `gettext 'Failed to download $p'`)"
		exit
	fi
done

# 8. extract package
for p in $MAIN_PKG $HELP_PKG $LANG_PKG; do
	echo $(eval echo `gettext 'Extracting $p'`)
	tar -xf ${p}.${PKGFORMAT}
done

# 9. extract debs
echo $(gettext 'Extracting DEBs ...')
find -name "*.deb" -exec undeb -x '{}' . \;

# 10. post-extract steps
echo $(gettext 'Post-extract steps')
if [ -L usr/local/bin/libreoffice${VERSION} ]; then
	# 4.4.x, must do more work
	mv usr/local/bin usr
	for p in usr/local/share/*; do
		mv $p usr/share
	done
	rmdir usr/local/share usr/local
fi
ln -sfv libreoffice${VERSION} usr/bin/libreoffice
ln -sfv libreoffice${VERSION} usr/bin/soffice
for app in base calc draw impress math printeradmin startcenter writer; do
	case $app in
		base|calc) cat="Database;Spreadsheet;" ;;
		draw|impress|math) cat="RasterGraphics;" ;;
		printeradmin|startcenter|writer) cat="WordProcessor;" ;;
	esac
	sed -i -e "s/^Categories=/Categories=${cat}/" opt/libreoffice${VERSION}/share/xdg/${app}.desktop
done

# 11. build SFS
echo $(gettext 'Building SFS')
mkdir squashfs-root
mv etc opt usr squashfs-root
mksquashfs squashfs-root libreoffice-$LATEST-$LANGPACK.sfs -noappend $COMP_MODE

# 12. cleanup
find -maxdepth 1 -type d | sed '1 d' | xargs rm -rf
rox .
echo $(gettext 'Done.')
